#!/bin/python3
# vim: set sts=4 ts=4 sw=4 ai expandtab:
# Copyright (C) 2026 Osamu Aoki <osamu@debian.org>
#
# This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of
# the GNU General Public License version 2 or later.
#
#Raw popularity-contest results

# inst = number of people who installed this package
# vote = number of people who use this package regularly
# old = number of people who installed, but don't use this package regularly
# recent = number of people who upgraded this package recently
# no-files = number of people whose entry didn't contain enough information (atime and ctime were 0)
# 
# # # of packages ~70K ( all-packages "74838")
# normal popcon     # = per #/1000
# too coarse for limited scope apps
#                                    74000   7400     740     74     7
# fine   popcon  per #/max*1000       1000    100      10      1     0 (base packages)
# fine   popcon1 per #/max*10000     10000   1000     100     10     1
# fine   popcon2 per #/max*100000   100000  10000    1000    100    10 (special packages)
#
# Shell script with bc etc. for floating point is nightmare
#
# Number of significant digits = 2
import sys
import re

def str_sig2(n):
    if [ n >= 1000 ]:
        s = "999"
    elif [ n >= 10 ]:
        s = "{:.0f}".format(n)
    elif [ n >= 1 ]:
        s = "{:.1f}".format(n)
    elif [ n >= 0.1 ]:
        s = "{:.2f}".format(n)
    else:
        s = "{:.3f}".format(n)
    return s

#r_data = re.compile(r"(?P<lead>.+?)\S+(?P<package>.+)")
#r_data = re.compile(r"(?P<lead>.+)\S+(?P<package>.+)\S+.*")
r_pattern2_data = re.compile(r"^(?P<lead>\S+)\s+(?P<data>\S+)\s*")
r_pattern6_data = re.compile(r"^(?P<lead>\S+)\s+(?P<package>\S+)\s+(?P<vote>\S+)\s+(?P<old>\S+)\s+(?P<new>\S+)\s+(?P<nofile>\S+)\s*")
#r_data = re.compile(r".*")


if __name__ == '__main__':

    # Process first line
    submissions = int(sys.argv[1])
#
    for i, line in enumerate(sys.stdin.read().splitlines()):
        data2 = r_pattern2_data.fullmatch(line)
        data6 = r_pattern6_data.fullmatch(line)
        if data6 is not None:
            lead = data6.group('lead')
            if lead != "Package:":
                print("# ERROR @{} : {}".format(i, line))
            else:
                package = data6.group('package')
                vote = int(data6.group('vote'))
                old = int(data6.group('old'))
                new = int(data6.group('new'))
                nofile = int(data6.group('nofile'))
                inst = vote + old + new + nofile
                package_normal = package.lower().translate(str.maketrans("0123456789","abcdefghij", r" !#$%()=-~^|\/+*,.?;:@`\"'&><"))
                inst_normal = inst / submissions * 1000
                vote_normal = vote / submissions * 1000
                if inst_normal < 0.01:
                    print('<!ENTITY pop-{} "I: NONE">'.format(package_normal))
                elif (vote * 100) > inst and vote > 0:
                    if inst_normal > 10:
                        print('<!ENTITY pop-{} "V:{:.0f}, I:{:.0f}">'.format(package_normal, vote_normal, inst_normal))
                    elif inst_normal > 1:
                        print('<!ENTITY pop-{} "V:{:.1f}, I:{:.1f}">'.format(package_normal, vote_normal, inst_normal))
                    else:
                        print('<!ENTITY pop-{} "V:{:.2f}, I:{:.2f}">'.format(package_normal, vote_normal, inst_normal))
                else:
                    if inst_normal > 10:
                        print('<!ENTITY pop-{} "I:{:.0f}">'.format(package_normal, inst_normal))
                    elif inst_normal > 1:
                        print('<!ENTITY pop-{} "I:{:.1f}">'.format(package_normal, inst_normal))
                    else:
                        print('<!ENTITY pop-{} "I:{:.2f}">'.format(package_normal, inst_normal))



#!/usr/bin/env python3
# vim: set sw=4 sts=4 expandtab:

"""
colspec.py - set colspec in XML to correct values

Copyright (C) 2009 Osamu Aoki, GPL
"""

import sys
import re

VERSION = "1.0.1"

if __name__ == "__main__":
    table_id = ""
    tgroup_cols = 0

    for line in sys.stdin.readlines():
        line = line[0:-1]
        if re.match('.*table id="', line):
            table_id = re.sub(r'.*table id="([^"]*)".*', r"\1", line)
        #            print "table id =" + table_id
        elif re.match(".*<title>", line):
            title = re.sub(r".*<title>([^<]*)</title>.*", r"\1", line)
        #            print "title =" + title
        elif re.match('.*tgroup cols="', line):
            tgroup_cols = int(re.sub(r'.*tgroup cols="([^"]*)".*', r"\1", line))
            #            print "tgroup_cols =", tgroup_cols
            cols = 0
            if tgroup_cols == 2:
                colwidth = (40, 60)
            elif tgroup_cols == 3:
                colwidth = (30, 30, 40)
            elif tgroup_cols == 4:
                colwidth = (20, 12, 6, 62)
            elif tgroup_cols == 5:
                colwidth = (20, 12, 6, 12, 50)
            elif tgroup_cols == 6:
                colwidth = (20, 12, 6, 12, 12, 38)
            elif tgroup_cols == 7:
                colwidth = (20, 12, 6, 12, 12, 12, 26)
            elif tgroup_cols == 8:
                colwidth = (20, 12, 6, 6, 6, 6, 6, 26)
            # List of exceptions
            if table_id == "thenumericmodefoinchmodbcommands":
                colwidth = (20, 80)
            elif table_id == "listofnotablesysupsforfileaccess":
                colwidth = (20, 80)
            elif table_id == "listofnotablesysommandexecutions":
                colwidth = (20, 80)
            elif table_id == "listoftypesoftimestamps":
                colwidth = (20, 80)
            elif table_id == "listofnotablesysupsforfileaccess":
                colwidth = (20, 80)
            elif table_id == "listofnotablepinpinningtechnique":
                colwidth = (20, 80)
            elif table_id == "listofkeyboardreigurationmethods":
                colwidth = (20, 80)
            elif table_id == "listoffilesystemalusagescenarios":
                colwidth = (20, 80)
            elif table_id == "thereplacementexpression":
                colwidth = (30, 70)
            elif table_id == "thekeybindingsofmc":
                colwidth = (30, 70)
            elif table_id == "listofkeywebsiteaspecificpackage":
                colwidth = (30, 70)
            elif table_id == "listofspecialdevicefiles":
                colwidth = (30, 20, 50)
            elif table_id == "listofdebianarchivearea":
                colwidth = (20, 20, 60)
            elif table_id == "listofviewsforaptitude":
                colwidth = (20, 20, 60)
            elif table_id == "listofrunlevelsationoftheirusage":
                colwidth = (20, 20, 60)
            elif table_id == "listofsshauthenttocolsandmethods":
                colwidth = (20, 40, 40)
            elif table_id == "listofterminologornetworkdevices":
                colwidth = (20, 20, 25, 35)
            elif table_id == "tableofkeywordsundicatefonttypes":
                colwidth = (25, 25, 25, 25)
            elif table_id == "listofinsecureanservicesandports":
                colwidth = (25, 25, 25, 25)
            elif table_id == "therelationshipbsuiteandcodename":
                colwidth = (25, 25, 25, 25)
            elif table_id == "theumaskvalueexamples":
                colwidth = (20, 25, 25, 30)
            elif table_id == "dimportantconfigilesforpam_unixi":
                colwidth = (20, 20, 10, 10, 40)
            elif table_id == "listoftoolstogeneratepassword":
                colwidth = (15, 15, 10, 15, 45)
            elif table_id == "listofnetworkaddressranges":
                colwidth = (10, 40, 30, 10, 10)

        if re.match('.*table pgwide="', line):
            print(re.sub(r'(.*table pgwide=)"0"(.*)', r'\1"1"\2', line))
        elif re.match('.*colspec colwidth="', line):
            x = str(colwidth[cols])
            cols += 1
            print(
                re.sub(r'(.*colspec colwidth=").*', r"\1", line)
                + str(x)
                + "*"
                + re.sub(r'.*colspec colwidth="[^"]*(".*)', r"\1", line)
            )
        else:
            print(line)



#!/usr/bin/env python3
# vim: set sts=4 expandtab:
"""
po4a-trim

Copyright (C) 2009 Osamu Aoki, GPL

Design: Trim po file depending on mode

Add blank line at the end of po
"""

import sys, os, re

VERSION = '1.0.1'

if __name__ == '__main__':

    po4a_mode = 'NORMAL'
    po4a_mode_old = 'NORMAL'
    # 'NORMAL' for header/comment which is dropped.
    # 'MSGID' for English original part
    # 'MSGSTR' for translated text part
    msgid = ''
    msgstr = ''

    for line in sys.stdin.readlines():
        line=line[0:-1]
        if re.match("^[ \t]*$", line):
            # blank line
            line = ""
            if po4a_mode != 'NORMAL':
                po4a_mode_old = po4a_mode
            po4a_mode = 'NORMAL'
        elif re.match("^#", line):
            # comment line
            line = ""
            if po4a_mode != 'NORMAL':
                po4a_mode_old = po4a_mode
            po4a_mode = 'NORMAL'
        elif re.match("^msgid", line):
            line = re.sub(r"^msgid +\"(.*)\" *$",r'\1',line)
            if po4a_mode != 'MSGID':
                po4a_mode_old = po4a_mode
            else:
                print("error @ " + line)
            po4a_mode = 'MSGID'
        elif re.match("^msgstr", line):
            line = re.sub(r"^msgstr +\"(.*)\" *$",r'\1',line)
            if po4a_mode != 'MSGSTR':
                po4a_mode_old = po4a_mode
            else:
                print("error @ " + line)
            po4a_mode = 'MSGSTR'
        elif re.match("^\"", line):
            line = re.sub(r"^\"(.*)\" *$",r'\1',line)
            # po4a_mode_old and po4a_mode are the same
        #
        if po4a_mode == 'MSGID':
            msgid = msgid + line
        if po4a_mode == 'MSGSTR':
            msgstr = msgstr + line
        if po4a_mode == 'NORMAL' and po4a_mode_old == 'MSGSTR':
            if msgid != "" and  msgstr != "":
                print("msgid \"" + msgid + "\"")
                print("msgstr \"" + msgstr + "\"\n")
            #
            msgid = ''
            msgstr = ''
            po4a_mode_old = 'NORMAL'





#!/bin/python3
# vim: set sts=4 ts=4 sw=4 ai expandtab:
# Copyright (C) 2026 Osamu Aoki <osamu@debian.org>
#
# This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of
# the GNU General Public License version 2 or later.
#
# $ apt list "~n^fcitx5.*" "~n^ibus.*" | bin/popconhtml.py
#
#  NEEDS: all-popcon-results.txt"
#
import sys
import re

#<tr>
#  <td style="text-align: left">
#    <a class="ulink" href="http://packages.debian.org/sid/ibus"> ibus </a>
#  </td>
#  <td style="text-align: left">
#    <a class="ulink" href="http://qa.debian.org/popcon-graph.php?packages=ibus">V:219, I:264</a>
#  </td>
#  <td style="text-align: left">
#    <a class="ulink" href="https://tracker.debian.org/pkg/ibus">1874</a>
#  </td>
#  <td style="text-align: left"> input method framework using dbus </td>
#</tr>
def output_tr(package, vote, inst, extra):
    print('<tr>')
    print('<td style="text-align: left">{} </td>'.format(package))
    if inst < 0.01:
        print('<td style="text-align: right"> --- </td> <td style="text-align: right"> --- </td>')
    elif (vote * 100) > inst and vote > 0:
        print('<td style="text-align: right"> {:.2f} </td> <td style="text-align: right"> {:.2f} </td>'.format(vote, inst))
    else:
        print('<td style="text-align: right"> --- </td> <td style="text-align: right"> {:.2f} </td>'.format(inst))
    if extra:
        print('<td style="text-align: left"><a class="ulink" href="https://bugs.debian.org/cgi-bin/pkgreport.cgi?{}"> bugs.d.o link (bin) </a> </td>'.format(package))
        print('<td style="text-align: left"><a class="ulink" href="https://qa.debian.org/popcon-graph.php?packages={}"> popcon-graph link (bin) </a> </td>'.format(package))
        print('<td style="text-align: left"><a class="ulink" href="https://packages.debian.org/sid/{}"> packages.d.o link (src) </a> </td>'.format(package))
        print('<td style="text-align: left"><a class="ulink" href="https://tracker.debian.org/pkg/{}"> tracker.d.o link (src) </a> </td>'.format(package))
    print('</tr>')

#r_data = re.compile(r"(?P<lead>.+?)\S+(?P<package>.+)")
#r_data = re.compile(r"(?P<lead>.+)\S+(?P<package>.+)\S+.*")
r_pattern2_data = re.compile(r"^(?P<lead>\S+)\s+(?P<data>\S+)\s*")
r_pattern6_data = re.compile(r"^(?P<lead>\S+)\s+(?P<package>\S+)\s+(?P<vote>\S+)\s+(?P<old>\S+)\s+(?P<new>\S+)\s+(?P<nofile>\S+)\s*")
#r_data = re.compile(r".*")


if __name__ == '__main__':

    # Process first line
    submissions = 0

    if len(sys.argv) > 0:
        extra = True
    extra = False
    packages = []
    for pkg in sys.stdin.read().splitlines():
        #print("pkg = '{}'".format(pkg.strip()))
        packages.append(pkg.strip().split('/')[0])
    print(
'''
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Package summary</title>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  </head>
  <body>
              <table class="table" summary="Debian binary package information" border="1">
                <colgroup>
                  <col style="text-align: left"/>
                  <col style="text-align: right"/>
                  <col style="text-align: right"/>
'''
          )
    if extra:
        print(
'''
                  <col style="text-align: left"/>
                  <col style="text-align: left"/>
                  <col style="text-align: left"/>
                </colgroup>
                <thead>
                  <tr>
                    <th style="text-align: left"> deb </th>
                    <th style="text-align: right"> Vote </th>
                    <th style="text-align: right"> Install </th>
                    <th style="text-align: left"> bts link </th>
                    <th style="text-align: left"> popcon-graph link </th>
                    <th style="text-align: left"> packages.d.o </th>
                    <th style="text-align: left"> tracker.d.o </th>
                  </tr>
                </thead>
                <tbody>
'''
            )
    with open("all-popcon-results.txt", "r") as fd:
        for i, line in enumerate(fd.read().splitlines()):
            data2 = r_pattern2_data.fullmatch(line)
            data6 = r_pattern6_data.fullmatch(line)
            if data6 is not None:
                lead = data6.group('lead')
                if lead != "Package:":
                    print("# ERROR @{} : {}".format(i, line))
                else:
                    package = data6.group('package').strip()
                    if package in packages:
                        vote0 = int(data6.group('vote'))
                        old0 = int(data6.group('old'))
                        new0 = int(data6.group('new'))
                        nofile0 = int(data6.group('nofile'))
                        inst0 = vote0 + old0 + new0 + nofile0
                        inst = inst0 / submissions * 1000
                        vote = vote0 / submissions * 1000
                        output_tr(package, vote, inst, extra)
            elif data2 is not None:
                lead = data2.group('lead')
                if lead == "Submissions:":
                    submissions = int(data2.group('data'))
    print(
'''
                </tbody>
  </body>
</html>
'''
          )




#!/usr/bin/env python3
# vim: set sts=4 expandtab:
"""
asc2title - convert an AsciiDoc titles

Copyright (C) 2009 Osamu Aoki, GPL
"""

import sys, os, re, string

VERSION = '1.0.1'

if __name__ == '__main__':

    for line in sys.stdin.readlines():
        line=line[0:-1]
        if line[0:1] == "=":
            line = re.sub(r'^=+ +([^ ].*[^ ]) *=*',r'\1',line)
            print(line)
#    entityname=pop-$(echo $package |tr "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789" "abcdefghijklmnopqrstuvwxyzabcdefghij"| \
#               tr -d " \!#\$%()=\-~^|\\/+*,.?;:@\`\"'&><")
# 32 char


